#!/usr/bin/env python

# Rss Notify
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.
#
# Authors: Isaiah Heyer (freshapplepy@gmail.com)

# Copyright (C) 2009 Isaiah Heyer

import feedparser
import os
import gobject

from xml.etree import ElementTree as ET
from xml.dom.minidom import parseString


class FeedManager(gobject.GObject):

    __gsignals__ = {
            "feed-list-changed": (gobject.SIGNAL_RUN_FIRST,
                    gobject.TYPE_NONE, [])
                    }

    def __init__(self, opml_filename):

        gobject.GObject.__init__(self)

        self.FEED_ATTRIB_TEMPLATE = {"title":"", "text":"",
                "description":"", "type":"rss", "htmlUrl":"", "xmlUrl":""}

        self.opml_file = opml_filename
        self.tree = None

        if os.path.exists(self.opml_file):
            
            self.import_feeds(self.opml_file)

        else:
            self.tree = self.build_tree()

    def build_tree(self):

        root = ET.Element("opml", {"version":"1.0"})

        head = ET.SubElement(root, "head")
        title = ET.SubElement(head, "title")
        title.text = "Rss Notify Feed List"

        body = ET.SubElement(root, "body")

        tree = ET.ElementTree(root)

        return tree

    def add_feed(self, xmlUrl, category=None):
        
        xmlUrl = xmlUrl.strip()
        
        try:
            f = feedparser.parse(xmlUrl)
            dummy = f.feed.title
        except Exception, data:
            print data
            return False


        title = f.feed.title
        text = f.feed.title
        description = f.feed.title
        otype = "rss"
        htmlUrl = f.feed.link
        xmlUrl = f.url
        
        new_feed_info = {"title":title, "text":text, "description":description,
                "type":otype, "htmlUrl":htmlUrl, "xmlUrl":xmlUrl}

        e = ET.Element("outline", new_feed_info)

        if category != None:
            category.append(e)
        else:
            body = self.get_feed_outlines()
            body.append(e)

        self.emit("feed-list-changed")
        return e

        
    def add_category(self, name, parent):
        """add_category(name, parent
        
        Adds a new feed category.
        """
        
        title = name
        text = name
        description = name
        otype = "folder"
        
        new_cat_info = {"title":title, "text":text,
                "description":description, "type":otype}

        e = ET.Element("outline", new_cat_info)
        if parent != None:
            parent.append(e)
        else:
            body = self.get_feed_outlines()
            body.append(e)

        return e


    def remove_feed(self, rem_element):
        """Removes rem_element from internal feed list

        """

        root = self.tree.getroot()

        def trav(element, remove_element):
            if remove_element in element.getchildren():
                element.remove(remove_element)
            else:
                for child in element.getchildren():
                    trav(child, remove_element)

        trav(root, rem_element)

        self.emit("feed-list-changed")
        return True


    def get_feed_urls(self):
        """Returns a list of all feeds urls"""
        Url_list = []

        def trav(element):
            xmlUrl = element.get("xmlUrl", None)
            if xmlUrl:
                Url_list.append(xmlUrl)
            for child in element:
                trav(child)

        gen = trav(self.tree.getroot())
        
        return Url_list

    def get_feed_outlines(self):

        for child in self.tree.getroot():
            if child.tag == "body":
                return child

    def import_feeds(self, filename):
        
        new_tree = ET.parse(filename)

        if self.tree:
            for child in self.tree.getroot():
                if child.tag == "body":
                    body = child
            
            
            for child in new_tree.getroot():
                if child.tag == "head":
                    for sub in child:
                        if sub.tag == "title":
                            t = sub.text

            ## Create new ET.Element 

            new = {}
            new['title'] = t
            new['text'] = t
            new['description'] = t
            new['type'] = "folder"
            new['expanded'] = "false"

            imported = ET.SubElement(body, "outline", new)
            for child in new_tree.getroot():
                if child.tag == "head":
                    for sub in child:
                        if sub.tag == "title":
                            sub.text = "Rss Notify Feed List"

                if child.tag == "body":
                    for sub in child:
                        if sub.tag == "outline":
                            child.remove(sub)
                            imported.append(sub)
                            
        else:
            self.tree = new_tree
            root = self.tree.getroot()
            for child in root:
                for sub in child:
                    if sub.tag == "title":
                        sub.text = "Rss Notify Feed List"

        ## In case outline element does not have a 
        ## "type" attrib
        def add_type_attrib(element):
            if element.tag == "outline":
                if not element.get("type", None):
                    if element.getchildren():
                        element.set("type","folder")
                    else:
                        element.set("type","rss")

            for child in element:
                add_type_attrib(child)


        add_type_attrib(self.tree.getroot())
        
        self.save_feeds()

        self.emit("feed-list-changed")
        return True


    def export_feeds(self, filename):

        def indent(elem, level=0):
            i = "\n" + level*"  "
            if len(elem):
                if not elem.text or not elem.text.strip():
                    elem.text = i + "  "
                for e in elem:
                    indent(e, level+1)
                    if not e.tail or not e.tail.strip():
                        e.tail = i + "  "
                if not e.tail or not e.tail.strip():
                    e.tail = i
            else:
                if level and (not elem.tail or not elem.tail.strip()):
                    elem.tail = i

        indent(self.tree.getroot())
        self.tree.write(filename)


    def save_feeds(self):
        self.export_feeds(self.opml_file)


